package sun.awt;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringBufferInputStream;
import java.util.Iterator;
import java.io.PrintStream;
import java.io.ByteArrayOutputStream;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Properties;
import sun.util.logging.PlatformLogger;

final class DebugSettings
{
    private static final PlatformLogger log;
    static final String PREFIX = "awtdebug";
    static final String PROP_FILE = "properties";
    private static final String[] DEFAULT_PROPS;
    private static DebugSettings instance;
    private Properties props;
    private static final String PROP_CTRACE = "ctrace";
    private static final int PROP_CTRACE_LEN;
    
    static void init() {
        if (DebugSettings.instance != null) {
            return;
        }
        NativeLibLoader.loadLibraries();
        (DebugSettings.instance = new DebugSettings()).loadNativeSettings();
        return;
    }
    
    private DebugSettings() {
        this.props = new Properties();
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                DebugSettings.access$000(DebugSettings.this);
                return null;
            }
        });
    }
    
    private synchronized void loadProperties() {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                DebugSettings.access$100(DebugSettings.this);
                DebugSettings.access$200(DebugSettings.this);
                DebugSettings.access$300(DebugSettings.this);
                return null;
            }
        });
        if (DebugSettings.log.isLoggable(PlatformLogger.Level.FINE)) {
            DebugSettings.log.fine("DebugSettings:\n{0}", new Object[] { this });
        }
    }
    
    @Override
    public String toString() {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        PrintStream printStream = new PrintStream(out);
        for (String s : this.props.stringPropertyNames()) {
            printStream.println(s + " = " + this.props.getProperty(s, ""));
        }
        return new String(out.toByteArray());
    }
    
    private void loadDefaultProperties() {
        try {
            for (int i = 0; i < DebugSettings.DEFAULT_PROPS.length; ++i) {
                StringBufferInputStream inStream = new StringBufferInputStream(DebugSettings.DEFAULT_PROPS[i]);
                this.props.load(inStream);
                inStream.close();
            }
        }
        catch (IOException ex) {}
    }
    
    private void loadFileProperties() {
        String pathname = System.getProperty("awtdebug.properties", "");
        if (pathname.equals("")) {
            pathname = System.getProperty("user.home", "") + File.separator + "awtdebug" + "." + "properties";
        }
        File file = new File(pathname);
        try {
            this.println("Reading debug settings from '" + file.getCanonicalPath() + "'...");
            FileInputStream inStream = new FileInputStream(file);
            this.props.load(inStream);
            inStream.close();
        }
        catch (FileNotFoundException ex) {
            this.println("Did not find settings file.");
        }
        catch (IOException ex2) {
            this.println("Problem reading settings, IOException: " + ex2.getMessage());
        }
    }
    
    private void loadSystemProperties() {
        Properties properties = System.getProperties();
        for (String s : properties.stringPropertyNames()) {
            String property = properties.getProperty(s, "");
            if (s.startsWith("awtdebug")) {
                this.props.setProperty(s, property);
            }
            else {
                continue;
            }
        }
    }
    
    public synchronized boolean getBoolean(String s, boolean b) {
        return this.getString(s, String.valueOf(b)).equalsIgnoreCase("true");
    }
    
    public synchronized int getInt(String s, int i) {
        return Integer.parseInt(this.getString(s, String.valueOf(i)));
    }
    
    public synchronized String getString(String str, String defaultValue) {
        return this.props.getProperty("awtdebug." + str, defaultValue);
    }
    
    private synchronized List<String> getPropertyNames() {
        LinkedList<String> list = new LinkedList();
        Iterator<String> iterator = this.props.stringPropertyNames().iterator();
        while (iterator.hasNext()) {
            list.add(((String)iterator.next()).substring("awtdebug".length() + 1));
        }
        return list;
    }
    
    private void println(Object o) {
        if (DebugSettings.log.isLoggable(PlatformLogger.Level.FINER)) {
            DebugSettings.log.finer(o.toString());
        }
    }
    
    private synchronized native void setCTracingOn(boolean p0);
    
    private synchronized native void setCTracingOn(boolean p0, String p1);
    
    private synchronized native void setCTracingOn(boolean p0, String p1, int p2);
    
    private void loadNativeSettings() {
        this.setCTracingOn(this.getBoolean("ctrace", false));
        LinkedList list = new LinkedList();
        for (String s : this.getPropertyNames()) {
            if (s.startsWith("ctrace") && s.length() > DebugSettings.PROP_CTRACE_LEN) {
                list.add(s);
            }
            else {
                continue;
            }
        }
        Collections.sort(list);
        for (String s2 : list) {
            String substring = s2.substring(DebugSettings.PROP_CTRACE_LEN + 1);
            int index = substring.indexOf(64);
            substring = ((index != -1) ? substring.substring(0, index) : substring);
            String s3 = (index != -1) ? substring.substring(index + 1) : "";
            int boolean1 = this.getBoolean(s2, false);
            if (s3.length() == 0) {
                this.setCTracingOn((boolean)boolean1, substring);
            }
            else {
                this.setCTracingOn((boolean)boolean1, substring, Integer.parseInt(s3, 10));
            }
        }
    }
    
    static void access$000(DebugSettings debugSettings) {
        debugSettings.loadProperties();
    }
    
    static void access$100(DebugSettings debugSettings) {
        debugSettings.loadDefaultProperties();
    }
    
    static void access$200(DebugSettings debugSettings) {
        debugSettings.loadFileProperties();
    }
    
    static void access$300(DebugSettings debugSettings) {
        debugSettings.loadSystemProperties();
    }
    
    static {
        log = PlatformLogger.getLogger("sun.awt.debug.DebugSettings");
        DEFAULT_PROPS = new String[] { "awtdebug.assert=true", "awtdebug.trace=false", "awtdebug.on=true", "awtdebug.ctrace=false" };
        DebugSettings.instance = null;
        PROP_CTRACE_LEN = "ctrace".length();
    }
}
